@@ -28,18 +28,14 @@ static void print_modalias(struct l_hwdb *hwdb, const char *format, ...)
 	vprintf(format, args);
 	va_end(args);
 
-	printf("\n");
-
 	va_start(args, format);
 	entries = l_hwdb_lookup_valist(hwdb, format, args);
 	va_end(args);
 
 	for (entry = entries; entry; entry = entry->next)
-		printf(" %s=%s\n", entry->key, entry->value);
+		fprintf(stderr, " %s=%s\n", entry->key, entry->value);
 
 	l_hwdb_lookup_free(entries);
-
-	printf("\n");
 }
 
 static void check_entry(const char *modalias, struct l_hwdb_entry *entries,
@@ -58,20 +54,20 @@ static void check_entry(const char *modalias, struct l_hwdb_entry *entries,
 	}
 }
 
-int main(int argc, char *argv[])
+static void test_hwdb(const void *data)
 {
 	struct l_hwdb *hwdb;
 	struct hwdb_stats stats = { 0 };
+	bool result;
 
 	hwdb = l_hwdb_new_default();
-	if (!hwdb) {
-		printf("hwdb.bin not loaded\n");
-		return 0;
-	}
+	assert(hwdb);
 
-	l_hwdb_foreach(hwdb, check_entry, &stats);
-	printf("Found %d aliases with %d total entries\n\n",
-	       stats.aliases, stats.entries);
+	result = l_hwdb_foreach(hwdb, check_entry, &stats);
+	assert(result);
+
+	fprintf(stderr, "Found %d aliases with %d total entries\n",
+					       stats.aliases, stats.entries);
 
 	/* Bluetooth Interest Group Inc. */
 	print_modalias(hwdb, "OUI:000F79");
@@ -86,6 +82,13 @@ int main(int argc, char *argv[])
 	print_modalias(hwdb, "sdio:c02");
 
 	l_hwdb_unref(hwdb);
+}
 
-	return 0;
+int main(int argc, char *argv[])
+{
+	l_test_init(&argc, &argv);
+
+	l_test_add("hwdb", test_hwdb, NULL);
+
+	return l_test_run();
 }
